//============================================================================
//  TLV2543.c                   Allen Marincak              Ver 1.0   Mar 2009
//
//      Driver and API for the TLV2543 11 channel 12 bit ADC device.
//
//  Copyright (c) 2009, Allen Marincak       See end of file for terms of use.
//============================================================================
//
// There are 11 12bit unipolar analog channels. It is possible to combine some
// bipolar operation but this driver us written exclusively for unipolar use. 
//
//
//                        +-----+  +-----+                      VCC = 3.3Vdc
//              Ain 0 ---[|1    +--+   20|]--- VCC
//                        |              |      
//              Ain 1 ---[|2           19|]--- EOC   -> 
//                        |              |   
//              Ain 2 ---[|3           18|]--- CLK   <-  
//                        |              |   
//              Ain 3 ---[|4           17|]--- Din   <- 
//                        |              |   
//              Ain 4 ---[|5           16|]--- Dout  -> 
//                        |              |                    VCC
//              Ain 5 ---[|6           15|]--- /CS   <-        |
//                        |              |                     |
//              Ain 6 ---[|7           14|]---{REF+}-----------+   close
//                        |              |                     |   as
//              Ain 7 ---[|8           13|]---{REF-}----+--||--+   possible
//                        |              |              |
//              Ain 8 ---[|9           12|]--- Ain 10   |  .33u Ceramic
//                        |              |              |     or
//                GND ---[|10          11|]--- Ain 9    |  .33u Tantalum
//                        +--------------+              |     NOT
//                                                      |   an Aluminum 
//      Include a .1u ceramic capacitor across          |   Electrolytic
//      chip's  Vcc and Gnd as close as possible        |
//      to the chip.                                   GND
//
//
//  The reference filter cap can be larger if .33u isn't handy, but much
//  larger is of no benefit (other than being handy if its all you got).
//
//   With the above :
//          Vref = Vcc = 3.3 V        giving 0.0008056640625 v / bit  
//          ( round it off to .8mV /bit for simple integer math)
//          ~= .8mV / bit is about .7% off, thus integer math will be
//          accurate to about 1% (acceptable +/- ?)
//
// ----------------------------------------------------------------------------
//  USING THE TLV2543 API
//
//  There are two ways to use this module, acquisition mode and manual mode.
//  The Acquisition mode will launch acquire() on a new COG to continuously
//  and sequentially sample all 11 channels at the full rate (~1082 samples
//  per second for each channel). The manual mode does not launch a cog and 
//  you must manually call for samples. The advantage to the manual mode is
//  that it does not use another cog. It is the slower of the two modes 
//  because it must wait for an ADC conversion to complete (~84uS) before
//  returning. The advantage of the acquisition mode is that you just read a
//  sample stored in a buffer. It returns nearly instantaneously, there is no
//  waiting required.
//
//  ACQUISITION MODE
//      To start acquisition call the TLV2543_start() function then call the 
//      TLV2543_getchan() function to get the channel data you want. This runs 
//      a new cog to constantly sample the ADC channels and place the conver-
//      sions in the readings[] array, ready to be read at any time.
//
//  MANUAL MODE
//      To start the manual mode call the TLV2543_init() function then call
//      the TLV2543_sample() function to start an ADC conversion and receive
//      the sample data. No extra cog is required, but it takes longer to get
//      ADC data (about 84uS vs less than 1uS using acquisition mode).
//
//  ADC SAMPLE RATE
//      At 80Mhz an analog channel is sampled in 84uS, all 11 channels are 
//      sampled in 924uS so in effect each channel is read every 924uS or 
//      about 1082 times a second.
//
// ============================================================================

#include <propeller.h>

static void acquire( void );

unsigned long   cog = -1;       // cog acquisition mode is running on
long            stack[20];      // stack space acquisition cog
unsigned short  readings[11];   // array to hold the ADC readings
unsigned long   pin_eoc, pin_clk, pin_din, pin_dout, pin_cs; 
  
//----------------------------------------------------------------------------
// TLV2543_init()                                   MANUAL MODE INITIALIZATION
//
//  Used to initialize the TLV2543 for manual use. ADC channels are read using
//  the TLV2543_sample() function.
//
//  The function arguments are the pins used by the propeller, they are named
//  after the pins the connect to on the TLV2543.
//
//  There is no return value.
//----------------------------------------------------------------------------
void TLV2543_init( char p_eoc, char p_clk, char p_din, char p_dout, char p_cs )
{
    pin_eoc  = 1 << p_eoc;
    pin_clk  = 1 << p_clk;
    pin_din  = 1 << p_din;
    pin_dout = 1 << p_dout;
    pin_cs   = 1 << p_cs;
    
    DIRA |=  pin_cs;                    // Set TLV2543 CS Pin to output
    OUTA |=  pin_cs;                    // Set TLV2543 CS pin high to disable device
    DIRA |=  pin_din;                   // Set TLV2543 DIN Pin to output
    OUTA &= ~pin_din;                   // Set TLV2543 DIN pin low
    DIRA |=  pin_clk;                   // Set TLV2543 CLK Pin to output
    OUTA &= ~pin_cs;                    // Set TLV2543 CLK pin low    
}    


//----------------------------------------------------------------------------
// TLV2543_start()                             ACQUISITION MODE INITIALIZATION
//
//  Used to initialize the TLV2543 and start the acquire() function on a new
//  COG. This is the acquisition mode startup call. ADC channels are read
//  using the TLV2543_getchan() function.
//
//  The function arguments are the pins used by the propeller, they are named
//  after the pins the connect to on the TLV2543.
//
//  The return value is the cog started or -1 if it failed to start.
//----------------------------------------------------------------------------
unsigned long TLV2543_start( char p_eoc, char p_clk, char p_din, char p_dout, char p_cs )
{
    TLV2543_init( p_eoc, p_clk, p_din, p_dout, p_cs );
    cog = cognew( acquire, &stack[19] );
    return( cog );
}    


//----------------------------------------------------------------------------
// TLV2543_stop()
//
//  Stops the acquisition cog and frees it up.
//----------------------------------------------------------------------------
void TLV2543_stop( void )
{
    if ( cog != -1 ) 
    {
        COGSTOP( cog );
        cog = -1;
    }     
}
    

//----------------------------------------------------------------------------
// TLV2543_getchan()
//
//  This function is used only for the acquisition mode to retrieve an ADC
//  reading for a channel from the buffer that the samples are stored in.
//
//  The calling parameter is the ADC channel number ( 0 - 10 ).
//
//  The return is the current channel reading.
//----------------------------------------------------------------------------
unsigned short TLV2543_getchan( char p_ch )
{
    return( readings[p_ch] );
}


//----------------------------------------------------------------------------
// TLV2543_sample()
//
//  This function is used in manual mode to start an ADC conversion and get
//  the result. It is also used by the acquire function in acquisition mode
//  for the same purpose. You must not use this function if the acquisition
//  mode is in effect.
//
//  The calling parameter is the ADC channel number ( 0 - 10 ).
//
//  The return is the channel reading.
//----------------------------------------------------------------------------
unsigned short TLV2543_sample( char p_ch )
{
    unsigned long chan, bit, chan_idx, idx, peoc;
    unsigned short val;
   
    OUTA &= ~pin_cs;                        // Set TLV2543 CS pin low to enable device
   
    chan = p_ch;
    peoc = pin_eoc;                         // must use local variables for WAITPEQ()
    bit = 0x800;
    chan <<= 8;                             // 12bit, MSBfirst, unipolar

    for ( idx = 0; idx < 12; idx++ )        // clock out instructions
    {
      if ( chan & bit ) OUTA |= pin_din;    // setup data bit
      else OUTA &= ~pin_din;

      OUTA |= pin_clk;                      // toggle clock high        
      bit >>= 1;                            // shift for next bit (here for a delay too)        
      OUTA &= ~pin_clk;                     // toggle clock low
    }
        
    asm( "nop\n nop\n nop\n nop" );         // short delay for EOC to go low
    WAITPEQ( peoc, peoc );                  // wait for EOC high
    val = 0;                                // clear, but also used as delay here
  
    for ( idx = 0; idx < 12; idx++ )        // clock in data
    {
        OUTA |= pin_clk;                    // toggle clock high  
        val <<= 1;                          // shift result to read next bit
        if ( INA & pin_dout ) val |= 1;     // read bit
        OUTA &= ~pin_clk;                   // toggle clock low
    }
    
    OUTA |= pin_cs;                         // Set TLV2543 CS pin high to disable device
     
    return( val );                          // return sample
}


//----------------------------------------------------------------------------
// acquire()
//
//  This method is launched on a new cog and rapidly cycles through all of the
//  ADC channels and stores their readings in a buffer for later retrieval
//  with the TLV2543_getchan() function. This is an internal function only, 
//  you do not call this function directly.
//----------------------------------------------------------------------------
static void acquire( void )
{
    char chan_idx;

    DIRA |=  pin_cs;                    // Set TLV2543 CS Pin to output
    OUTA |=  pin_cs;                    // Set TLV2543 CS pin high to disable device
    DIRA |=  pin_din;                   // Set TLV2543 DIN Pin to output
    OUTA &= ~pin_din;                   // Set TLV2543 DIN pin low
    DIRA |=  pin_clk;                   // Set TLV2543 CLK Pin to output
    OUTA &= ~pin_cs;                    // Set TLV2543 CLK pin low

    chan_idx = 0;
             
    for(;;)
    {  
        readings[chan_idx] = TLV2543_sample(chan_idx);  // save reading to array    
        chan_idx++;                                     // next channel
        if ( chan_idx == 11 ) chan_idx = 0;
    }
}                      


//============================================================================
//                        TERMS OF USE: MIT License                                                       
//============================================================================
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
//                                       
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//============================================================================
