//============================================================================
//  TLV2543_test.c              Allen Marincak              Ver 1.0   Mar 2009
//
//      Demo for TI TLV2543 12-Bit ADC Driver.
//
//  Copyright (c) 2009, Allen Marincak       See end of file for terms of use.
//============================================================================
//
// This program must run at 80Mhz.
//
//  There are two ways to compile this program depending on how you set the
//  ACQUIRE_MODE define (below). If it is set to 1 another COG is launched
//  that reads and converts each channel at about 1082 times a second and
//  stores the readings in a buffer tht you can retrieve virtually
//  instantaneously at any time. If the ACQUIRE_MODE define is set to zero
//  you must call for a chanel conversion one at a time, this will take about
//  84uS to complete.
//
// Propeller IO Utilization
//
//   Pin  0     - Out - Status LED
// 
//   Pin  1     - In
//   Pin  2     - In
//   Pin  3     - In
//   Pin  4     - In
//   Pin  5     - In
//   Pin  6     - In
//   Pin  7     - In
//
//   Pin  8     - In  - TLV2543 EOC
//   Pin  9     - Out - TLV2543 CLK
//   Pin 10     - Out - TLV2543 Din
//   Pin 11     - In  - TLV2543 Dout
//   Pin 12     - Out - TLV2543 /CE
//
//   Pin 13     - In
//   Pin 14     - In
//   Pin 15     - In
//   Pin 16     - In
//   Pin 17     - In
//   Pin 18     - In
//   Pin 19     - In
//   Pin 20     - In
//   Pin 21     - In
//   Pin 22     - In
//   Pin 23     - In
//   Pin 24     - In
//   Pin 25     - In
//   Pin 26     - In
//   Pin 27     - In
//
//   Pin 28     - Out - I2C Clock   ( not used in demo )
//   Pin 29     - I/O - I2C Data    ( not used in demo )
//
//   Pin 30     - Out - serial TX
//   Pin 31     - In  - serial RX
//
// ----------------------------------------------------------------------------
//   COGS used: 3 or 4
//
//      * LMM Kernel
//      * C Library
//      * main
//      * TLV2534 (only if in Acquire mode ... see AQUIRE_MODE define below )
// ============================================================================

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <propeller.h>

#include "FdSerial.h"
#include "tlv2543.h"
#include "timers.h"


#define serial_rx   31          // serial Rx line
#define serial_tx   30          // serial tx line
  
#define i2cSDALine  29          // I2C SDA is on Propeller pin A29
#define i2cSCLLine  28          // I2C SCL is on Propeller pin A28
  
#define tlv_eoc     8           // TVL2543 pins
#define tlv_clk     9           // TVL2543 pins
#define tlv_din     10          // TVL2543 pins
#define tlv_dout    11          // TVL2543 pins
#define tlv_cs      12          // TVL2543 pins

#define status_LED  0           // hearbeat LED


#define ACQUIRE_MODE 1          // 0 = individual sample calls (no COG launched)
                                // 1 = acquisition mode on separate COG

//----------------------------------------------------------------------------
//  main()
//----------------------------------------------------------------------------  
void main( void )
{
    unsigned long tmr1, ctr, idx, LED;

    LED = 1 << status_LED;
    DIRA |= LED;                        // Set Status LED pin to output

#if ( ACQUIRE_MODE )
    TLV2543_start( tlv_eoc, tlv_clk, tlv_din, tlv_dout, tlv_cs );   // init and launch a COG
#else
    TLV2543_init( tlv_eoc, tlv_clk, tlv_din, tlv_dout, tlv_cs );    // just init
#endif    
    FdSerial_start( serial_rx, serial_tx, 0, 9600 );
    TMRS_Start( 10 );

    tmr1 = TMRS_Register();

    ctr = 0;
    TMRS_Set( tmr1, 2 );                // set timer to expire in .2 second
    
    for(;;)
    {
        if ( TMRS_IsClr( tmr1 ) )
        {
            TMRS_Set( tmr1, 2 );
            OUTA ^= LED;
            ctr++;
        }
    
        if ( ctr == 10 )                // every 2 sec
        {
            ctr = 0;

            for( idx = 0; idx < 11; idx++ )
            {
#if ( ACQUIRE_MODE )            
                FdSerial_hex( TLV2543_getchan( idx ), 4 );  // read stored data
#else                
                FdSerial_hex( TLV2543_sample( idx ), 4 );   // start a conversion and wait for result
#endif                
                FdSerial_tx( 32 );
            }
            
            FdSerial_str( "\r\n" ); 
        }
    }
}            

//============================================================================
//                        TERMS OF USE: MIT License                                                       
//============================================================================
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
//                                       
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//============================================================================
   