//============================================================================
//  i2c_test.c                    Allen Marincak                      Mar 2009
//
//  Copyright (c) 2009, Allen Marincak       See end of file for terms of use.
//============================================================================
//
//  This program tests and demos the C version of the I2C driver. It has been
//  modified (generalized) from the original and ported to the C language
//  (Imagecraft ICCV7). It uses the 'C' FDSerial Driver (c) Steve Denson 
//  (included in zip file).
//
//  This I2C driver has been tested on various I2C devices and this demo will
//  drive:
//          - the Devantech LCD03 I2C Character LCD display 
//          - the Devantech SRF10 I2C Ultrasonic Ranger
//
//  A simple Heartbeat LED is driven from the main loop using a timer object.
//============================================================================

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <propeller.h>

#include "i2c.h"
#include "FdSerial.h"
#include "timers.h"

#define status_led  0

#define i2c_clk     28
#define i2c_dat     29
#define serial_tx   30
#define serial_rx   31

#define serial_baud 57600

#define lcd_addr    0xC6
#define srf10_addr  0xE0

unsigned long   status_led_pin = 1 << status_led;

//----------------------------------------------------------------------------
//  main()
//----------------------------------------------------------------------------
void main( void )
{
    unsigned long tmr1, tmr2, rng;
    unsigned char rng_lo, rng_hi;
    unsigned char lcd, srf;
    
    
    DIRA |= status_led_pin;                 // set status LED pin output
    OUTA &= ~status_led_pin;                // set status LED pin low (LED off)
    
    TMRS_Start( 10 );                       // Start timer cog
    tmr1 = TMRS_Register();                 // register a timer
    tmr2 = TMRS_Register();                 // register a timer
    
    FdSerial_start(serial_rx,serial_tx,0,serial_baud);  // start serial driver
    
    I2C_Init( i2c_dat, i2c_clk );
    
    TMRS_Set( tmr2, 40 );
    while ( !TMRS_IsClr( tmr2 ) );
    
    FdSerial_str( "Scanning for I2C devices ...\r\n" );

    lcd = I2C_IsDevPresent( lcd_addr );
    FdSerial_str( "  - Devantech LCD03 at address 0xC6 is " );
    if ( lcd ) FdSerial_str( "PRESENT.\r\n" );
    else FdSerial_str( "NOT PRESENT.\r\n" ); 
    
    srf = I2C_IsDevPresent( srf10_addr );
    FdSerial_str( "  - Devantech SRF10 at address 0xE0 is " );
    if ( srf ) FdSerial_str( "PRESENT.\r\n" );
    else FdSerial_str( "NOT PRESENT.\r\n" ); 
    
    if ( lcd )
    {  
        //------------------------------------------------------------------------
        // Devantech LCD02/03  I2C/Serial LCD    
        //------------------------------------------------------------------------   
        I2C_WriteDevReg( lcd_addr, 0, 12 );     // 0 = cmd reg   12 = clear screen
        I2C_WriteDevRegStr( lcd_addr, 0, "I2C Driver Test Prog" );
        I2C_WriteDevReg3( lcd_addr, 0, 3, 3, 1 );
        I2C_WriteDevRegStr( lcd_addr, 0, " ICCV7 'C' Compiler " );
    }        
   
    TMRS_Set( tmr2, 20 );                    // start 2 second count
    TMRS_Set( tmr1, 2 );                     // start .2 second count
 
    for(;;)
    {
        if ( TMRS_IsClr( tmr1 ) )
        {
            TMRS_Set( tmr1, 2 );             // restart .2 second count
            if ( OUTA & status_led_pin )
                OUTA &= ~status_led_pin;
            else
                OUTA |= status_led_pin;
        }
        
        if ( srf )
        {
            if ( TMRS_IsClr( tmr2 ) )
            {
                TMRS_Set( tmr2, 20 );                   // restart 2 sec timer
                I2C_WriteDevReg( srf10_addr, 0, 81 );   // 0 = cmd reg   81 = range in cm
                       
                while ( I2C_ReadDevReg( srf10_addr, 0 ) == 255 );
            
                rng_lo = I2C_ReadDevReg( srf10_addr, 3 );
                rng_hi = I2C_ReadDevReg( srf10_addr, 2 );

                rng = ( (long)rng_hi << 8 ) + (long)rng_lo;

                FdSerial_str( "Range " );
                FdSerial_dec( rng );
                FdSerial_str( " cm\r\n" );                        
            }
        }
    }
}


//============================================================================
//                        TERMS OF USE: MIT License                                                       
//============================================================================
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
//                                       
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//============================================================================
         