/* =========================================================================
 * MCP3202 :: 12-bit, 1-channel, analog digital converter driver 
 * -------------------------------------------------------------------------
 * File:    mcp3201.c
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-28)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/28: V001 - Initial Release   
 * ========================================================================= */

#include <propeller.h>

#include "mcp3201.h"

/* -------------------------------------------------------------------------
 * mcp3201_setup () - setup the device (automatically calls mcp3201_reset to 
 * initialize everything)  
 * ------------------------------------------------------------------------- */
void mcp3201_setup (
	MCP3201 *dev,
	unsigned char cs,
    unsigned char clk,
    unsigned char dout)
{
	// pin configuration
	dev->CSPin = cs;
    dev->CLKPin = clk;
    dev->DOUTPin = dout;
    
    // data read mask 
    dev->internal.mask = 1 << dev->DOUTPin;
    
    // converter default 
    dev->internal.ctype = NO_CONVERSION;
    dev->internal.cf = (MCP3201_Converter) NULL;
    dev->internal.min = 0.0;
    dev->internal.max = 0.0;
    
    // setup pins 
    mcp3201_reset (dev);
}

/* -------------------------------------------------------------------------
 * mcp3201_reset () - setup all pins
 * ------------------------------------------------------------------------- */
void mcp3201_reset (MCP3201 *dev)
{
	SET (DIRA, dev->CSPin);						// CS is output
    SET (OUTA, dev->CSPin);						// and idles high 
    
    SET (DIRA, dev->CLKPin);					// clock is output
    CLR (OUTA, dev->CLKPin);					// and idles low
    
    CLR (DIRA, dev->DOUTPin);					// data-output is input
}

/* -------------------------------------------------------------------------
 * mcp3201_set_range () - set conversion range  
 * ------------------------------------------------------------------------- */
void mcp3201_set_range (
	MCP3201 *dev,
	float min,
    float max)
{
	dev->internal.min = min;
    dev->internal.max = max; 
    dev->internal.ctype = RANGE;
}

/* -------------------------------------------------------------------------
 * mcp3201_set_converter () - set custom converter  
 * ------------------------------------------------------------------------- */
void mcp3201_set_converter (
	MCP3201 *dev, 
    MCP3201_Converter cf)
{
	dev->internal.cf = cf;
    dev->internal.ctype = CUSTOM_FUNCTION;
}

/* -------------------------------------------------------------------------
 * mcp3201_raw () - clear any set converter  
 * ------------------------------------------------------------------------- */
void mcp3201_raw (
	MCP3201 *dev)
{
	dev->internal.ctype = NO_CONVERSION;
}

/* -------------------------------------------------------------------------
 * mcp3201_convert () - read and convert   
 * ------------------------------------------------------------------------- */
float mcp3201_convert (
	MCP3201 *dev)
{
	unsigned short reading = mcp3201_read (dev);
    float result;
    
    switch (dev->internal.ctype)
    {
    	case NO_CONVERSION:
    		result = reading;
            break;
        case RANGE:    	
        	result = 
            	     (dev->internal.max - dev->internal.min) 
                   * (float) reading / (float) ADC_RANGE
                   + dev->internal.min;
            break;
    	case CUSTOM_FUNCTION:
        	result = dev->internal.cf (reading);
            break;
    }
    
    return result;
}

/* -------------------------------------------------------------------------
 * mcp3201_read () - read measured value
 * ------------------------------------------------------------------------- */
unsigned short mcp3201_read (
	MCP3201 *dev)
 {
	unsigned short reading = 0; // result
    int i; 
    
    // open communication (CS line low)
    CLR (OUTA, dev->CSPin);
    
    // sample and hold clocks 
    i = SAMPLE_HOLD_CLOCKS;
    for (i = 0; i < SAMPLE_HOLD_CLOCKS; i++)
    {
        SET (OUTA, dev->CLKPin);
        CLR (OUTA, dev->CLKPin);
    }
    
    // receive the conversion result 
    for (i = 1; i <= DATA_SIZE; i++)
    {
        SET (OUTA, dev->CLKPin);
        CLR (OUTA, dev->CLKPin); // next clock
     	reading <<= 1; // shift existing result one bit left 
        reading |= ((INA & dev->internal.mask) != 0); // read next bit
    }
    
    // terminate communication (CS line high)
    SET (OUTA, dev->CSPin);
    
    return reading;
}

/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
