/* =========================================================================
 * MCP3202 :: 12-bit, 1-channel, test application for MCP3201 driver
 * -------------------------------------------------------------------------
 * File:    mcp3201_test.c
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-28)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/28: V001 - Initial Release   
 * ========================================================================= */

#include <propeller.h>
#include <asio.h>
#include <stdio.h>

#include "mcp3201.h"

/* -------------------------------------------------------------------------
 * Prototypes
 * ------------------------------------------------------------------------- */
int main (void);			// main function
void example1 (void);		// reading raw values
void example2 (void);		// conversion using range
void example3 (void);		// conversion using custom function
                            
float pressure_altitude (unsigned short); // custom converter to convert 
                                          // raw readings in to altitude

/* -------------------------------------------------------------------------
 * Serial terminal
 * ------------------------------------------------------------------------- */
#define BAUD_RATE 57600

/* -------------------------------------------------------------------------
 * Numerical constants
 * ------------------------------------------------------------------------- */
#define SUCCESS 0
#define READINGS 10

#define ADC_REF_VOLTAGE 3.3

/* -------------------------------------------------------------------------
 * Pins used 
 * ------------------------------------------------------------------------- */
#define CLOCK_PIN	26
#define DOUT_PIN    25
#define CS_PIN      24

/* -------------------------------------------------------------------------
 * main () --- entry point
 * ------------------------------------------------------------------------- */
int main (void)
{
    asio_init (BAUD_RATE);
    
	printf ("\n\n");
    printf ("========================\n");
	printf ("MCP3201 Test Application\n");
    printf ("========================\n\n");

    printf ("-----------------------------------------------------------\n");
	printf ("Example 1: raw readings\n");
    printf ("-----------------------------------------------------------\n");
    example1 ();
    
    printf ("-----------------------------------------------------------\n");
	printf ("Example 2: conversion using range\n");
    printf ("-----------------------------------------------------------\n");
    example2 ();
    
    printf ("-----------------------------------------------------------\n");
	printf ("Example 3: conversion using custom converter function\n");
    printf ("-----------------------------------------------------------\n");
    example3 ();
    
	return SUCCESS;
}
                            
/* -------------------------------------------------------------------------
 * example1 () --- make 10 readings, no conversion
 * ------------------------------------------------------------------------- */
void example1 ()
{
	MCP3201 dev;
    int iReading;
    unsigned short reading;
    
	// setup the device 
    mcp3201_setup (
    	&dev,
    	CS_PIN,
        CLOCK_PIN,
        DOUT_PIN);
        
	// perform and display the readings 
    for (iReading = 1; iReading <= READINGS; iReading++)
    {
        reading = mcp3201_read (&dev);
        printf ("#%02d: reading = 0x%03X\n", iReading, reading);
    }
    
    printf ("\n\n");
}

/* -------------------------------------------------------------------------
 * example2 () --- make 10 readings and convert them into specific range
 * ------------------------------------------------------------------------- */
void example2 ()
{
	MCP3201 dev;
    int iReading;
    float voltage;
    
	// setup the device 
    mcp3201_setup (
    	&dev,
    	CS_PIN,
        CLOCK_PIN,
        DOUT_PIN);
        
    // set the range converter 
    mcp3201_set_range (&dev, 0.0, ADC_REF_VOLTAGE);
        
	// obtain and display the converted readings 
    for (iReading = 1; iReading <= READINGS; iReading++)
    {
       	voltage = mcp3201_convert (&dev);
        printf ("#%02d: voltage = %8.5f [V]\n", iReading, voltage);
    }
    
    printf ("\n\n");
}

/* -------------------------------------------------------------------------
 * example3 () --- make 10 reading and convert them using custom converter
 * ------------------------------------------------------------------------- */
void example3 ()
{
	MCP3201 dev;
    int iReading;
    float reading;
    
	// setup the device 
    mcp3201_setup (
    	&dev,
    	CS_PIN,
        CLOCK_PIN,
        DOUT_PIN);
        
    // set the custom converter 
    mcp3201_set_converter (&dev, pressure_altitude);
                
	// perform and display the readings 
    for (iReading = 1; iReading <= READINGS; iReading++)
    {
        reading = mcp3201_convert (&dev);
        printf ("#%02d: altitude = %12.4f [ft]\n", iReading, reading);
    }
    
    printf ("\n\n");
}

/* ------------------------------------------------------------------------
 * Additional numeric constants for example 4
 * ------------------------------------------------------------------------ */
#define REF_VOLTAGE 3.3

#define MIN_PRESSURE 15.0
#define MAX_PRESSURE 115.0

#define SEA_LEVEL_PRESSURE 101.325

/* ------------------------------------------------------------------------
 * pressure_altitude () --- custom conversion function that takes raw 
 * reading of the voltage from the pressure sensor and converts it to 
 * altitude (using a lookup table and linear interpolation) 
 * ------------------------------------------------------------------------ */

#define PRESS_TABLE_LENGTH 55
#define LOWEST_ALTITUDE -4000
#define ALTITUDE_STEP 1000

static float PressTable[PRESS_TABLE_LENGTH] = // lookup table
{
	116.855, 112.803, 108.866, 105.042, 101.327, 97.718, 94.214, 90.813, 87.512, 
     84.309,  81.201,  78.187,  75.264,  72.430, 69.683, 67.021, 64.442, 61.944,  
     59.525,  57.183,  54.916,  52.723,  50.601, 48.548, 46.564, 44.646, 42.792, 
     41.001,  39.272,  37.602,  35.989,  34.434, 32.933, 31.486, 30.090, 28.745, 
     27.449,  26.201,  24.999,  23.843,  22.730, 21.659, 20.630, 19.640, 18.690,
     17.777,  16.900,  16.059,  15.252,  14.478, 13.736, 13.026, 12.345, 11.694, 
     11.070
};

float pressure_altitude (unsigned short raw)
{
	float voltage;
    float pressure;
    float altitude;
	
    unsigned short iData;
        
	// convert raw reading into voltage
    voltage = REF_VOLTAGE * (float) raw / (float) ADC_RANGE;
    
    // convert voltage into pressure
    pressure = 
               voltage / REF_VOLTAGE 
             * (MAX_PRESSURE - MIN_PRESSURE) 
             + MIN_PRESSURE;
             
    // find the closest pressure in the table
    for (iData = 1; iData < PRESS_TABLE_LENGTH; iData++)
    {
    	if (pressure >= PressTable[iData] && pressure <= PressTable[iData - 1])
        {
            // the pressure is somewhere between iData - 1 and iData
            // table entries 
        	break;
        }
    }
    
    if (iData < PRESS_TABLE_LENGTH)
    {
        // compute the whole-step portion of the altitude
        altitude = LOWEST_ALTITUDE + (iData - 1) * ALTITUDE_STEP;
        
        // use linear interpolation to improve the result 
        altitude += 
                    (pressure - PressTable[iData])
                  / (PressTable[iData - 1] - PressTable[iData])
                  * ALTITUDE_STEP;
    }
              
    return altitude;
}

/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
