/* =========================================================================
 * MCP3204 :: 12-bit, 4-channel, analog digital converter driver 
 * -------------------------------------------------------------------------
 * File:    mcp3204.c
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-17)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/17: V001 - Initial Release   
 * ========================================================================= */

#include <propeller.h>

#include "mcp3204.h"

/* -------------------------------------------------------------------------
 * mcp3204_setup () - setup the device (automatically calls mcp3204_reset to 
 * initialize everything)  
 * ------------------------------------------------------------------------- */
void mcp3204_setup (
	MCP3204 *dev,
	unsigned char cs,
    unsigned char clk,
    unsigned char din,
    unsigned char dout, 
    EMode mode)
{
	// pin configuration
	dev->CSPin = cs;
    dev->CLKPin = clk;
    dev->DINPin = din;
    dev->DOUTPin = dout;
    
    // data read mask 
    dev->internal.mask = 1 << dev->DOUTPin;
    
    // converter default 
    dev->internal.ctype = NO_CONVERSION;
    dev->internal.cf = (MCP3204_Converter) NULL;
    dev->internal.min = 0.0;
    dev->internal.max = 0.0;
    
    // mode of operation
    dev->mode = mode;
        
    // setup pins 
    mcp3204_reset (dev);
}

/* -------------------------------------------------------------------------
 * mcp3204_reset () - setup all pins and precalculate control byte
 * ------------------------------------------------------------------------- */
void mcp3204_reset (MCP3204 *dev)
{
	SET (DIRA, dev->CSPin);						// CS is output
    SET (OUTA, dev->CSPin);						// and idles high 
    
    SET (DIRA, dev->CLKPin);					// clock is output
    CLR (OUTA, dev->CLKPin);					// and idles low
    
    SET (DIRA, dev->DINPin);					// data-input is output 
    CLR (OUTA, dev->DINPin);					// and idles low 
    
    CLR (DIRA, dev->DOUTPin);					// data-output is input

    // preliminary control byte calculation
    dev->internal.config = mcp3204_preconfig (dev->mode);    
}
/* -------------------------------------------------------------------------
 * mcp3204_preconfig () - precalculate the configuration (control byte)  
 * ------------------------------------------------------------------------- */
unsigned char mcp3204_preconfig (
	unsigned char mode)
{
	return START_BIT | mode;
}

/* -------------------------------------------------------------------------
 * mcp3204_set_range () - set conversion range  
 * ------------------------------------------------------------------------- */
void mcp3204_set_range (
	MCP3204 *dev,
	float min,
    float max)
{
	dev->internal.min = min;
    dev->internal.max = max; 
    dev->internal.ctype = RANGE;
}

/* -------------------------------------------------------------------------
 * mcp3204_set_converter () - set custome converter  
 * ------------------------------------------------------------------------- */
void mcp3204_set_converter (
	MCP3204 *dev, 
    MCP3204_Converter cf)
{
	dev->internal.cf = cf;
    dev->internal.ctype = CUSTOM_FUNCTION;
}

/* -------------------------------------------------------------------------
 * mcp3204_raw () - clear any set converter  
 * ------------------------------------------------------------------------- */
void mcp3204_raw (
	MCP3204 *dev)
{
	dev->internal.ctype = NO_CONVERSION;
}

/* -------------------------------------------------------------------------
 * mcp3204_convert () - read and convert   
 * ------------------------------------------------------------------------- */
float mcp3204_convert (
	MCP3204 *dev,
    EChannel channel)
{
	unsigned short reading = mcp3204_read (dev, channel);
    float result;
    
    switch (dev->internal.ctype)
    {
    	case NO_CONVERSION:
    		result = reading;
            break;
        case RANGE:    	
        	result = 
            	     (dev->internal.max - dev->internal.min) 
                   * (float) reading / (float) ADC_RANGE
                   + dev->internal.min;
            break;
    	case CUSTOM_FUNCTION:
        	result = dev->internal.cf (reading);
            break;
    }
    
    return result;
}

/* -------------------------------------------------------------------------
 * mcp3204_read () - read one channel 
 * ------------------------------------------------------------------------- */
unsigned short mcp3204_read (
	MCP3204 *dev,
    EChannel channel)
{
	unsigned short reading = 0; // result
    int i; 
    
	// add channel to precalculated config byte 
    unsigned char cbyte = dev->internal.config | channel;
    
    // open communication (CS line low)
    CLR (OUTA, dev->CSPin);
    
    // send the control byte
    for (i = CONTROL_BITS - 1; i >= 0; i--)
    {
    	// latch the value 
        OUTA |= (((cbyte & (1 << i)) != 0) << dev->DINPin);
               
        // clock it out 
        SET (OUTA, dev->CLKPin);
        CLR (OUTA, dev->CLKPin); 
        
        // reset DIN pin
        CLR (OUTA, dev->DINPin);    
    } 
    
    // one more clock cycle to complete SAMPLE&HOLD
    SET (OUTA, dev->CLKPin);
    CLR (OUTA, dev->CLKPin);
    
    // receive the conversion result 
    for (i = 1; i <= DATA_SIZE; i++)
    {
        SET (OUTA, dev->CLKPin);
        CLR (OUTA, dev->CLKPin); // next clock
     	reading <<= 1; // shift existing result one bit left 
        reading |= ((INA & dev->internal.mask) != 0); // read next bit
    }
    
    // terminate communication (CS line high)
    SET (OUTA, dev->CSPin);
    
    return reading;
}

/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
