/* =========================================================================
 * MCP3208 :: 12-bit, 8-channel, test application for MCP3208 ADC driver
 * -------------------------------------------------------------------------
 * File:    mcp3208_test.c
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-17)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/17: V001 - Initial Release   
 * ========================================================================= */

#include <propeller.h>
#include <asio.h>
#include <stdio.h>

#include "mcp3208.h"

/* -------------------------------------------------------------------------
 * Prototypes
 * ------------------------------------------------------------------------- */
int main (void);			// main function
void example1 (void);		// reading both channels in SINGLE mode
void example2 (void);		// reading in PSEUDODIFFERENTIAL mode
void example3 (void);		// reading in SINGLE mode with range conversion
void example4 (void);       // reading in PSEUDODIFFERENTIAL mode with 
                            // custom conversion
                            
float pressure_altitude (unsigned short); // custom converter to convert 
                                          // raw readings in to altitude

/* -------------------------------------------------------------------------
 * Serial terminal
 * ------------------------------------------------------------------------- */
#define BAUD_RATE 57600

/* -------------------------------------------------------------------------
 * Numerical constants
 * ------------------------------------------------------------------------- */
#define SUCCESS 0
#define READINGS 10
#define CHANNELS 8

#define ADC_REF_VOLTAGE 3.3

/* -------------------------------------------------------------------------
 * Pins used 
 * ------------------------------------------------------------------------- */
#define CS_PIN      27
#define CLOCK_PIN	26
#define DOUT_PIN    25
#define DIN_PIN     24

/* -------------------------------------------------------------------------
 * main () --- entry point
 * ------------------------------------------------------------------------- */
int main (void)
{
    asio_init (BAUD_RATE);
    
	printf ("\n\n");
    printf ("========================\n");
	printf ("MCP3208 Test Application\n");
    printf ("========================\n\n");

    printf ("-----------------------------------------------------------\n");
	printf ("Example 1: SINGLE mode, all channels, no conversion\n");
    printf ("-----------------------------------------------------------\n");
    example1 ();
    
    printf ("-----------------------------------------------------------\n");
	printf ("Example 2: PSEUDODIFF mode, all channels, no conversion\n");
    printf ("-----------------------------------------------------------\n");
    example2 ();
    
    printf ("-----------------------------------------------------------\n");
	printf ("Example 3: SINGLE mode, all channels, range conversion\n");
    printf ("-----------------------------------------------------------\n");
    example3 ();
    
    printf ("-----------------------------------------------------------\n");
	printf ("Example 4: PSEUDODIFF mode, all channels, custom conversion\n");
    printf ("-----------------------------------------------------------\n");
    example4 ();

	return SUCCESS;
}
                            
/* -------------------------------------------------------------------------
 * example1 () --- read all channels 10 times in single mode, no conversion
 * ------------------------------------------------------------------------- */
void example1 ()
{
	MCP3208 dev;
    int iReading;
    int iChannel;
    unsigned short reading;
    EChannel channels[CHANNELS] = {
        CHANNEL_0, 
        CHANNEL_1,
        CHANNEL_2,
        CHANNEL_3,
        CHANNEL_4,
        CHANNEL_5,
        CHANNEL_6,
        CHANNEL_7};
    
	// setup the device 
    mcp3208_setup (
    	&dev,
    	CS_PIN,
        CLOCK_PIN,
        DIN_PIN,
        DOUT_PIN,
        MODE_SINGLE);
        
	// perform and display the readings 
    for (iReading = 1; iReading <= READINGS; iReading++)
    {
    	for (iChannel = 0; iChannel < CHANNELS; iChannel++)
        {
        	reading = mcp3208_read (&dev, channels[iChannel]);
            printf (
            	"#%02d: channel = %1d, reading = 0x%03X\n",
            	iReading,
                iChannel,
                reading);
        }
        printf ("---------------------------------------------------------\n");
    }
    
    printf ("\n\n");
}

/* -------------------------------------------------------------------------
 * example2 () --- read all channels 10 times in pseudodifferential mode,
 * no conversion. Channel number indicates where the positive analog 
 * input is connected.
 * ------------------------------------------------------------------------- */
void example2 ()
{
	MCP3208 dev;
    int iReading;
    int iChannel;
    unsigned short reading;
    EChannel channels[CHANNELS] = {
        CHANNEL_0_1,
        CHANNEL_1_0,
        CHANNEL_2_3,
        CHANNEL_3_2,
        CHANNEL_4_5,
        CHANNEL_5_4,
        CHANNEL_6_7,
        CHANNEL_7_6};
    
	// setup the device 
    mcp3208_setup (
    	&dev,
    	CS_PIN,
        CLOCK_PIN,
        DIN_PIN,
        DOUT_PIN,
        MODE_DIFFERENTIAL);
        
	// perform and display the readings 
    for (iReading = 1; iReading <= READINGS; iReading++)
    {
    	for (iChannel = 0; iChannel < CHANNELS; iChannel++)
        {
        	reading = mcp3208_read (&dev, channels[iChannel]);
            printf (
            	"#%02d: channel = %1d, reading = 0x%03X\n",
            	iReading,
                iChannel,
                reading);
        }
        printf ("---------------------------------------------------------\n");
    }
    
    printf ("\n\n");
}

/* -------------------------------------------------------------------------
 * example3 () --- read all channels 10 times in single mode, perform range
 * conversion 
 * ------------------------------------------------------------------------- */
void example3 ()
{
	MCP3208 dev;
    int iReading;
    int iChannel;
    float reading;
    EChannel channels[CHANNELS] = {
        CHANNEL_0, 
        CHANNEL_1,
        CHANNEL_2,
        CHANNEL_3,
        CHANNEL_4,
        CHANNEL_5,
        CHANNEL_6,
        CHANNEL_7};
    
	// setup the device 
    mcp3208_setup (
    	&dev,
    	CS_PIN,
        CLOCK_PIN,
        DIN_PIN,
        DOUT_PIN,
        MODE_SINGLE);
        
    // set the range converter 
    mcp3208_set_range (
    	&dev,
        0.0,
        ADC_REF_VOLTAGE);
        
	// perform and display the readings 
    for (iReading = 1; iReading <= READINGS; iReading++)
    {
    	for (iChannel = 0; iChannel < CHANNELS; iChannel++)
        {
        	reading = mcp3208_convert (&dev, channels[iChannel]);
            printf (
            	"#%02d: channel = %1d, reading = %7.4f [V]\n",
            	iReading,
                iChannel,
                reading);
        }
        printf ("---------------------------------------------------------\n");
    }
    
    printf ("\n\n");
}

/* ------------------------------------------------------------------------
 * Additional numeric constants for example 4
 * ------------------------------------------------------------------------ */
#define REF_VOLTAGE 3.3

#define MIN_PRESSURE 15.0
#define MAX_PRESSURE 115.0

#define SEA_LEVEL_PRESSURE 101.325
#define PALT_EXPONENT 0.19025515825
#define PALT_COEFFICIENT (288.15 / 0.001990740740)

/* -------------------------------------------------------------------------
 * example4 () --- read all channels 10 times in pseudodifferential mode,
 * use custom function to convert readings into altitude 
 * (using the "pressure altitude" formula)
 * ------------------------------------------------------------------------- */
void example4 ()
{
	MCP3208 dev;
    int iReading;
    int iChannel;
    float reading;
    EChannel channels[CHANNELS] = {
        CHANNEL_0_1, 
        CHANNEL_1_0,
        CHANNEL_2_3,
        CHANNEL_3_2,
        CHANNEL_4_5,
        CHANNEL_5_4,
        CHANNEL_6_7,
        CHANNEL_7_6};
    
	// setup the device 
    mcp3208_setup (
    	&dev,
    	CS_PIN,
        CLOCK_PIN,
        DIN_PIN,
        DOUT_PIN,
        MODE_DIFFERENTIAL);
        
    // set the range converter 
    mcp3208_set_converter (
    	&dev,
        pressure_altitude);
        
	// perform and display the readings 
    for (iReading = 1; iReading <= READINGS; iReading++)
    {
    	for (iChannel = 0; iChannel < CHANNELS; iChannel++)
        {
        	reading = mcp3208_convert (&dev, channels[iChannel]);
            printf (
            	"#%02d: channel = %1d, reading = %7.4f [ft]\n",
            	iReading,
                iChannel,
                reading);
        }
        printf ("---------------------------------------------------------\n");
    }
    
    printf ("\n\n");
}

/* ------------------------------------------------------------------------
 * pressure_altitude () --- custom conversion function that takes raw 
 * reading of the voltage from the pressure sensor and converts it to 
 * altitude (using a lookup table and linear interpolation) 
 * ------------------------------------------------------------------------ */

#define PRESS_TABLE_LENGTH 55
#define LOWEST_ALTITUDE -4000
#define ALTITUDE_STEP 1000

static float PressTable[PRESS_TABLE_LENGTH] = // lookup table
{
	116.855, 112.803, 108.866, 105.042, 101.327, 97.718, 94.214, 90.813, 87.512, 
     84.309,  81.201,  78.187,  75.264,  72.430, 69.683, 67.021, 64.442, 61.944,  
     59.525,  57.183,  54.916,  52.723,  50.601, 48.548, 46.564, 44.646, 42.792, 
     41.001,  39.272,  37.602,  35.989,  34.434, 32.933, 31.486, 30.090, 28.745, 
     27.449,  26.201,  24.999,  23.843,  22.730, 21.659, 20.630, 19.640, 18.690,
     17.777,  16.900,  16.059,  15.252,  14.478, 13.736, 13.026, 12.345, 11.694, 
     11.070
};

float pressure_altitude (unsigned short raw)
{
	float voltage;
    float pressure;
    float altitude;
	
    unsigned short iData;
        
	// convert raw reading into voltage
    voltage = REF_VOLTAGE * (float) raw / (float) ADC_RANGE;
    
    // convert voltage into pressure
    pressure = 
               voltage / REF_VOLTAGE 
             * (MAX_PRESSURE - MIN_PRESSURE) 
             + MIN_PRESSURE;
             
    // find the closest pressure in the table
    for (iData = 1; iData < PRESS_TABLE_LENGTH; iData++)
    {
    	if (pressure >= PressTable[iData] && pressure <= PressTable[iData - 1])
        {
            // the pressure is somewhere between iData - 1 and iData
            // table entries 
        	break;
        }
    }
    
    if (iData < PRESS_TABLE_LENGTH)
    {
        // compute the whole-step portion of the altitude
        altitude = LOWEST_ALTITUDE + (iData - 1) * ALTITUDE_STEP;
        
        // use linear interpolation to improve the result 
        altitude += 
                    (pressure - PressTable[iData])
                  / (PressTable[iData - 1] - PressTable[iData])
                  * ALTITUDE_STEP;
    }
              
    return altitude;
}

/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
