//============================================================================
// timers.c                                         V1.03  A.Marincak Mar 2009
//
//  Copyright (c) 2009, Allen Marincak       See end of file for terms of use.
//============================================================================
//
// Starts a new cog to provide timing functionality.
//
// There are 8 timers available, the code running on the new cog simply
// decrements each timer value at the rate specified  in the start() call.
//
// Only the first object to create this needs to call the START method. That
// will launch it on a new COG, other objects can reference it in the OBJ
// section but do not need to start it, just call the public methods.
// ============================================================================

#include <propeller.h>

static void TMRS_Execute( void );

unsigned char   tmrs_used[8] = {0,0,0,0,0,0,0,0};   // bitfield signifying timers in use
unsigned short  timers[8]    = {0,0,0,0,0,0,0,0};   // timer values (16 bit)
long            tmr_stack[16];                      // stack for new COG
int             tmr_period;

//----------------------------------------------------------------------------
// TMRS_Start()
//
// Launches TIMER TASK on new cog, returns 0 on error, else the cog that was
// started up.
//    - period is the fractional part of a second to operate at
//      example:  
//                10 = 1/10th of a second
//                30 = 1/30th of a second
//               100 = 1/100th of a second
//----------------------------------------------------------------------------
int TMRS_Start( int period )
{
    int okay;

    tmr_period = period;    
    okay = cognew( TMRS_Execute, &tmr_stack[15] )+ 1;
    return( okay );
}      

  
//----------------------------------------------------------------------------
// TMRS_Register()
//
// Register a timer. Returns the first free timer (8 timers numbered 0 to 7)
// or returns -1 if none are available.
//----------------------------------------------------------------------------
int TMRS_Register( void )
{
    int idx;
    

    for ( idx = 0; idx < 8; idx++ )
    {
        if ( tmrs_used[idx] == 0 )
        {
            tmrs_used[idx] = 1;
            return( idx );
        }
    }

    return( -1 );
}


//----------------------------------------------------------------------------
// TMRS_Unregister()
//
// Unregisters a timer, frees it up for reuse.
//----------------------------------------------------------------------------
void TMRS_Unregister( int tmrid )
{
    if ( tmrs_used[tmrid] == 1 )
    {
        timers[tmrid] = 0;
        tmrs_used[tmrid] = 0;
    }
}

    
//----------------------------------------------------------------------------
// TMRS_Set()
//
// Sets a registered timer with a 16 bit value. 
//----------------------------------------------------------------------------
void TMRS_Set( int tmrid, unsigned short val )
{
    if ( tmrs_used[tmrid] == 1 )
        timers[tmrid] = val;
}


//----------------------------------------------------------------------------
// TMRS_IsClr()
//
// Checks the status of the registered timer.
// returns 1 if it has expired (zeroed)
//         0 if is still running
//----------------------------------------------------------------------------
int TMRS_IsClr( int tmrid )
{
    return ( timers[tmrid] ? 0 : 1 );
}


//----------------------------------------------------------------------------
// TMRS_Read()
//
// returns the current count of a registered timer.
//----------------------------------------------------------------------------
int TMRS_Read( int tmrid )
{
  return( timers[tmrid] );
}  

  
//----------------------------------------------------------------------------
// TMRS_Execute()
//
// new cog executes this, it just decrements the timers
//----------------------------------------------------------------------------
static void TMRS_Execute( void )
{
    unsigned long idx, interval_cnt, time_base;
  
    interval_cnt = CLKFREQ / tmr_period;
    time_base = CNT + interval_cnt;

    for (;;)
    {
        WAITCNT( time_base, interval_cnt );     // incs timebase by interval_cnt
        
        for ( idx = 0; idx < 8; idx++ )
        {
            if ( timers[idx] != 0 )
                timers[idx] -= 1;
        }
    }
}
    